@@ -1,5 +1,5 @@
 lambertW = function(z,b=0,maxiter=10,eps=.Machine$double.eps,min.imag=1e-9) {
-  badz = is.na(z)
+  badz = !is.finite(z)
   z.old = z
   z = z[!badz]
   if (any(round(Re(b)) != b))
@@ -21,8 +21,8 @@ lambertW = function(z,b=0,maxiter=10,eps=.Machine$double.eps,min.imag=1e-9) {
   v = v - log(v + as.numeric(v==0))
   ## choose strategy for initial guess
   ##
-  c = abs(z + exp(-1));
-  c = (c > 1.45 - 1.1*abs(b));
+  c = abs(z + exp(-1))
+  c = (c > 1.45 - 1.1*abs(b))
   c = c | (b*Im(z) > 0) | (!Im(z) & (b == 1))
   w = (1 - c)*w + c*v
   ## Halley iteration
@@ -33,14 +33,17 @@ lambertW = function(z,b=0,maxiter=10,eps=.Machine$double.eps,min.imag=1e-9) {
     f = (w != -1)
     t = f*t/(p*(w + f) - 0.5*(w + 2.0)*t/(w + f))
     w = w - t
-    if (any(is.na(t) | is.na(w))) {
-      ## FIXME: what to do here?
-      ## stop("NAs encountered in LambertW")
-      ## print(t)
-      ## print(w)
-    }
-    if (abs(Re(t)) < (2.48*eps)*(1.0 + abs(Re(w)))
-        && abs(Im(t)) < (2.48*eps)*(1.0 + abs(Im(w))))
+    ## if (any(is.na(t) | is.na(w))) {
+    ## FIXME: what to do here?
+    ## stop("NAs encountered in LambertW")
+    ## print(t)
+    ## print(w)
+    ## }
+    ok <- is.finite(t) & is.finite(w)
+    ## iterate until ALL values converged -- perhaps
+    ## inefficient
+    if (all(abs(Re(t[ok])) < (2.48*eps)*(1.0 + abs(Re(w[ok])))
+            & abs(Im(t[ok])) < (2.48*eps)*(1.0 + abs(Im(w[ok])))))
       break
   }
   if (n==maxiter) warning(paste("iteration limit (",maxiter,
@@ -128,13 +131,16 @@ curve3d <- function (expr, from=c(0,0), to=c(1,1), n = c(41,41), add = FALSE,
 }
 
 get.emdbook.packages <- function() {
-   pkglist = c("adapt","bbmle","chron","coda","ellipse","gplots","gtools","gdata",
-     "MCMCpack","odesolve","plotrix","R2WinBUGS","reshape","rgl",
+   pkglist = c("adapt","bbmle","chron",
+     "coda","ellipse","ggplot2",
+     "gplots","gtools","gdata",
+     "MCMCpack","deSolve","plotrix","R2WinBUGS","reshape","rgl",
      "scatterplot3d")
    inst.pkgs = rownames(installed.packages())
    newpkgs <- pkglist[!pkglist %in% inst.pkgs]
    if (length(newpkgs)>0) {
-     do.call("install.packages",list(pkglist))
+      install.packages(newpkgs)
+   ##  do.call("install.packages",list(pkglist))
    ## sapply(pkglist,install.packages)
    }
  }
@@ -335,84 +341,18 @@ ncredint <- function(pvec,npost,level=0.95,tol=0.01,verbose=FALSE) {
   if (verbose) return(r) else return(r[1:2])
 }
 
-## credible interval for a sample (e.g. Markov chain)
-##
-## OBSOLETE: HPDinterval [coda] is better
-##
-## qcredint <- function(x,level=0.95,tol=0.01,verbose=FALSE,...) {
-##   d = density(x,...)
-##   pvec = d$x
-##   npost = d$y
-##   dx = diff(pvec)[1]
-##   cumdist <- cumsum(npost)*dx
-##   midpt <- which.min(abs(cumdist-0.5))
-##   lims <- function(pdens) { ## find lower and upper values for which
-##     ## prob dens is closest to target value
-##     lower <- which.min(abs(npost[1:midpt]-pdens))
-##     upper <- which.min(abs(npost[(midpt+1):length(npost)]-pdens))+midpt
-##     c(lower,upper)
-##   }
-##   limarea <- function(pdens) {
-##     intlim <- lims(pdens)
-##     d <- sum(npost[intlim[1]:intlim[2]])*dx
-##     ##    cat(pdens,intlim,d,"\n")
-##     d
-##   }
-##   ## find credible interval
-##   v2 <- seq(0,max(npost),by=tol)
-##   vals <- sapply(v2,limarea)
-##   w <- which.min(abs(vals-level))
-##   r = c(pvec[lims(v2[w])],v2[w],limarea(v2[w]))
-##   names(r) = c("lower","upper","p","area")
-##   ## credible intervals; posterior density, area
-##   if (verbose) return(r) else return(r[1:2])
-## }
 
-HPDregionplot <- function(x,vars=1:2,h=c(1,1),n=50,lump=TRUE,prob=0.95,
-                          xlab=NULL,ylab=NULL,...) {
-  require("MASS") ## for kde2d
-  parnames <- if (class(x)=="mcmc.list") colnames(x[[1]]) else colnames(x)
-  if (is.character(vars)) {
-    vars <- match(vars,parnames)
-    if (any(is.na(vars))) stop("some variable names do not match")
-  }
-  varnames <- parnames[vars]
-  mult <- (class(x)=="mcmc.list" && !lump)
-  if (mult) stop("multiple chains without lumping not yet implemented")
-  if (class(x)=="mcmc.list") {
-    if (lump) var1 <- c(sapply(x,function(z)z[,vars[1]]))
-    else var1 <- lapply(x,function(z)z[,vars[1]])
-  } else var1 <- x[,vars[1]]
-  if (class(x)=="mcmc.list") {
-    if (lump) var2 <- c(sapply(x,function(z)z[,vars[2]]))
-    else var2 <- lapply(x,function(z)z[,vars[2]])
-  } else var2 <- x[,vars[2]]
-  if (!mult) {
-    post1 = kde2d(var1,var2,n=n,h=h)
-    ## post0 = post1
-  } else {
-    post1 = mapply(kde2d,var1,var2,MoreArgs=list(n=n))
-  }
-  dx = diff(post1$x[1:2])
-  dy = diff(post1$y[1:2])
-  sz = sort(post1$z)
-  c1 = cumsum(sz)*dx*dy
-  levels = sapply(prob,function(x) {approx(c1,sz,xout=1-x)$y})
-  ## meanvec <- c(mean(var1),mean(var2))
-  if (is.null(xlab)) xlab <- varnames[1]
-  if (is.null(ylab)) ylab <- varnames[2]
-  contour(post1$x,post1$y,post1$z,level=levels,
-          xlab=xlab,ylab=ylab,drawlabels=FALSE,...)
-  invisible(contourLines(post1$x,post1$y,post1$z,level=levels))
-}
 
 calcslice <- function(fit1,fit2,fn=fit1@minuslogl,range=c(-0.1,1.1),
                   n=400) {
     ## require(bbmle) 
     slicep = seq(range[1],range[2],length=n)
     slicepars = t(sapply(slicep,function(x) (1-x)*coef(fit1)+x*coef(fit2)))
+    if (!require(bbmle)) {
+      stop("need to install bbmle in order to use calcslice")
+    }
     ## FIXME: warning about parnames from R CMD check
-    if (is.null(parnames(fn))) {
+    if (is.null(bbmle::parnames(fn))) {
         v = apply(slicepars,1,function(x) do.call("fn",as.list(x)))
     } else { ## vector-argument function
         v = apply(slicepars,1,fn)
@@ -471,16 +411,6 @@ qchibarsq <- function(q,df=1,mix=0.5) {
   ##   mixtures?
 }
 
-## make an mcmc object out of an mcmc.list
-lump.mcmc.list <- function(x) {
-  x2 <- do.call("rbind",x)
-  mcpars <- sapply(x,attr,"mcpar")
-  class(x2) <- "mcmc"
-  if (var(mcpars[1,])>0 || var(mcpars[3,])>0)
-    stop("can't combine chains with unequal start/thin")
-  attr(x2,"mcpar") <- c(mcpars[1,1],sum(mcpars[2,]),mcpars[3,1])
-  x2
-}
 
 dmvnorm <- function (x, mu, Sigma, log = FALSE, tol = 1e-06) {
     if (is.vector(x)) 
@@ -683,55 +613,3 @@ trcoef = function(x,inverse=FALSE) {
   x
 }
 
-perturb.params = function(base,alt,which,
-  mult=FALSE,use.base=TRUE) {
-  if (!missing(alt)) {
-    chlist = mapply(
-      function(name,vals) {
-        x = lapply(vals,
-               function(z2) {
-                 if (mult) {
-                   base[[name]] = base[[name]]*z2}
-                 else base[[name]]=z2
-                 base
-               })
-        if (is.list(x)) x else list(x)
-      },
-      names(alt),alt)
-    chlist = unlist(chlist,recursive=FALSE)
-  }
-  if (use.base) {
-    chlist=append(list(base),chlist)
-  }
-  chlist
-}
-
-## copied from traceplot in coda package, but uses lattice
-traceplot.mcmc <- function (x, data = NULL, outer, aspect = "xy",
-                            default.scales = list(y=list(relation = "free")), 
-                            start = 1, thin = 1,
-                            main = attr(x, "title"), xlab = "",
-                            ylab="",
-                            plot.points = "rug", ...,
-                            subset = coda:::thinned.indices(x, start = start, 
-                              thin = thin))
-  ## would like to supply subset argument that would also
-  ## select specific variables
-  ## consider aspect="fill" as an alternative
-{
-    require(lattice)
-    if (!is.R()) {
-        stop("This function is not yet available in S-PLUS")
-    }
-    if (!missing(outer)) 
-        warning("specification of outer ignored")
-    data <- as.data.frame(x)
-    v <- seq(nrow(x))
-    form <- as.formula(paste(paste(lapply(names(data), as.name),
-        collapse = "+"), "~v"))
-    xyplot(form, data = data[subset, ], outer = TRUE, aspect = aspect, 
-           default.scales = default.scales, main = main, xlab = xlab,
-           ylab=ylab,
-           type="l",
-           plot.points = plot.points, ...)
-}
